@@ -344,6 +344,7 @@ class cmd_qbrowse(QBzrCommand):
     aliases = ['qbw']
 
     def _qbzr_run(self, revision=None, location=None):
+        Branch.open_containing(location or u'.')  # if there is no branch we want NotBranchError raised
         if revision is None:
             win = BrowseWindow(location = location)
         else:
@@ -1084,7 +1085,11 @@ class cmd_qrun(QBzrCommand):
         directory=None, category=None, execute=False):
         from bzrlib.plugins.qbzr.lib.run import QBzrRunDialog
         if parameters_list:
-            parameters = " ".join(parameters_list)
+            def quote_spaces(s):
+                if " " in s:
+                    s = '"%s"' % s
+                return s
+            parameters = " ".join([quote_spaces(i) for i in parameters_list])
         else:
             parameters = None
         if not command:
