@@ -50,12 +50,14 @@ lazy_import(globals(), '''
 from bzrlib import (
     osutils,
     urlutils,
+    ui,
 )
-from bzrlib.util.configobj import configobj
 from bzrlib.plugins.qbzr.lib import trace
 from bzrlib.workingtree import WorkingTree
 from bzrlib.transport import get_transport
 from bzrlib.lockdir import LockDir
+
+from bzrlib.plugins.qbzr.lib.compatibility import configobj
 ''')
 
 # standard buttons with translatable labels
@@ -113,6 +115,10 @@ class Config(object):
         if section not in self._configobj:
             self._configobj[section] = {}
         if value:
+            if not isinstance(value, (str,unicode)):
+                # [bialix 2011/02/11] related to bug #716384: if value is bool
+                # then sometimes configobj lost it in the output file
+                value = str(value)
             self._configobj[section][name] = value
         else:
             if name in self._configobj[section]:
@@ -127,6 +133,14 @@ class Config(object):
         except KeyError:
             return None
 
+    def get_option_as_bool(self, name, section=None):
+        # imitate the code from bzrlib.config to read option as boolean
+        # until we will switch to use bzrlib.config instead of our re-implementation
+        value_maybe_str_or_bool = self.get_option(name, section)
+        if value_maybe_str_or_bool not in (None, ''):
+            value = ui.bool_from_string(value_maybe_str_or_bool)
+            return value
+
     def set_section(self, name, values):
         self._load()
         self._configobj[name] = values
@@ -244,7 +258,7 @@ def _check_global_config_filename_valid(config):
         return not config.file_name == config_filename()
     else:
         return False
-    
+
 
 _qbzr_config = None
 def get_qbzr_config():
@@ -255,11 +269,12 @@ def get_qbzr_config():
     return _qbzr_config
 
 def get_branch_config(branch):
-    if branch is not None:
+    if branch: # we should check boolean branch value to support 2 fake branch cases: branch is None, branch is FakeBranch
         return branch.get_config()
     else:
         return get_global_config()
 
+
 class _QBzrWindowBase(object):
 
     def set_title(self, title=None):
@@ -339,8 +354,8 @@ class _QBzrWindowBase(object):
             self.resize(size.expandedTo(self.minimumSizeHint()))
         self._restore_size = size
 
-        is_maximized = config.get_option(name + "_window_maximized")
-        if is_maximized in ("True", "1"):
+        is_maximized = config.get_option_as_bool(name + "_window_maximized")
+        if is_maximized:
             self.setWindowState(QtCore.Qt.WindowMaximized)
         return config
 
@@ -550,7 +565,7 @@ def hookup_directory_picker(dialog, chooser, target, chooser_type):
             # Or a QLineEdit
             getter = target.text
             setter = target.setText
-        dir = getter()
+        dir = unicode(getter())
         if not os.path.isdir(dir):
             dir = ""
         dir = QtGui.QFileDialog.getExistingDirectory(dlg, caption, dir)
@@ -621,7 +636,7 @@ def get_set_encoding(encoding, branch):
                 'utf-8 will be used instead') % encoding)
             encoding = 'utf-8'
     else:
-        if branch is not None:
+        if branch: # we should check boolean branch value to support 2 fake branch cases: branch is None, branch is FakeBranch
             branch.get_config().set_user_option("encoding", encoding)
     return encoding
 
@@ -745,6 +760,15 @@ def fill_combo_with(combo, default, *iterables):
             done.add(item)
             combo.addItem(item)
 
+def show_shortcut_hint(action):
+    """Show this action's shortcut, if any, as part of the tooltip.
+    
+    Make sure to set the shortcut and tooltip *before* calling this.
+    """
+    shortcut = action.shortcut()
+    if shortcut and shortcut.toString():
+        toolTip = action.toolTip()
+        action.setToolTip("%s (%s)" % (toolTip, shortcut.toString()))
 
 def iter_saved_pull_locations():
     """ Iterate the 'pull' locations we have previously saved for the user.
@@ -972,16 +996,45 @@ def launchpad_project_from_url(url):
 
     @return: project name or None
     """
-    # The format ought to be scheme://host/~user-id/project-name/branch-name/
+    # The format ought to be one of the following:
+    #   scheme://host/~user-id/project-name/branch-name
+    #   scheme://host/+branch/project-name
+    #   scheme://host/+branch/project-name/series-name
+    # there could be distro branches, they are very complex,
+    # so we only support upstream branches based on source package
+    #   scheme://host/+branch/DISTRO/SOURCEPACKAGE
+    #   scheme://host/+branch/DISTRO/SERIES/SOURCEPACKAGE
+    #   scheme://host/+branch/DISTRO/POCKET/SOURCEPACKAGE
+    #   scheme://host/~USER/DISTRO/SERIES/SOURCEPACKAGE/BRANCHNAME
+    DISTROS = ('debian', 'ubuntu')
     from urlparse import urlsplit
     scheme, host, path = urlsplit(url)[:3]
     # Sanity check the host
-    if (host.find('bazaar.launchpad.net') >= 0 or
-        host.find('bazaar.launchpad.dev') >= 0):
+    if (host in ('bazaar.launchpad.net',
+                 'bazaar.launchpad.dev',
+                 'bazaar.qastaging.launchpad.net',
+                 'bazaar.staging.launchpad.net')):
         parts = path.strip('/').split('/')
-        if len(parts) == 3 and (parts[0].startswith('~')
-                                or (parts[0] in ('%2Bbranch', '+branch'))):
-            return parts[1]
+        if parts[0].startswith('~'):
+            if len(parts) == 3 and parts[1] not in DISTROS:
+                # scheme://host/~user-id/project-name/branch-name/
+                return parts[1]
+            elif len(parts) == 5 and parts[1] in DISTROS:
+                # scheme://host/~USER/DISTRO/SERIES/SOURCEPACKAGE/BRANCHNAME
+                return parts[-2]
+        elif parts[0] in ('%2Bbranch', '+branch'):
+            n = len(parts)
+            if n >= 2:
+                part1 = parts[1]
+                if n in (2,3) and part1 not in DISTROS:
+                    # scheme://host/+branch/project-name
+                    # scheme://host/+branch/project-name/series-name
+                    return part1
+                elif n in (3,4) and part1 in DISTROS:
+                    # scheme://host/+branch/DISTRO/SOURCEPACKAGE
+                    # scheme://host/+branch/DISTRO/SERIES/SOURCEPACKAGE
+                    # scheme://host/+branch/DISTRO/POCKET/SOURCEPACKAGE
+                    return parts[-1]
     return None
 
 
@@ -1001,121 +1054,10 @@ if MS_WINDOWS:
 else:
     shlex_split_unicode = _shlex_split_unicode_linux
 
-def get_icon(name):
+def get_icon(name, size=22):
     # TODO: Load multiple sizes
     # TODO: Try load from system theme
-    return QtGui.QIcon(":/22x22/%s.png" % name)
-
-
-class FindToolbar(QtGui.QToolBar):
-    
-    def __init__(self, window, text_edit, show_action):
-        QtGui.QToolBar.__init__(self, gettext("Find"), window)
-        self.text_edit = text_edit
-        if 0: self.text_edit = QtGui.QTextEdit()
-        self.show_action = show_action
-        
-        self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
-        self.setMovable (False)
-        
-        find_label = QtGui.QLabel(gettext("Find: "), self)
-        self.addWidget(find_label)
-        
-        self.find_text = QtGui.QLineEdit(self)
-        self.addWidget(self.find_text)
-        find_label.setBuddy(self.find_text)
-        
-        prev = self.addAction(get_icon("go-previous"), gettext("Previous"))
-        prev.setShortcut(QtGui.QKeySequence.FindPrevious)
-        
-        next = self.addAction(get_icon("go-next"), gettext("Next"))
-        next.setShortcut(QtGui.QKeySequence.FindNext)
-        
-        self.case_sensitive = QtGui.QCheckBox(gettext("Case sensitive"), self)
-        self.addWidget(self.case_sensitive)
-        self.whole_words = QtGui.QCheckBox(gettext("Whole words"), self)
-        self.addWidget(self.whole_words)
-        
-        close_find = QtGui.QAction(self)
-        close_find.setIcon(self.style().standardIcon(
-                                        QtGui.QStyle.SP_DialogCloseButton))
-        self.addAction(close_find)
-        close_find.setShortcut((QtCore.Qt.Key_Escape))
-        close_find.setShortcutContext(QtCore.Qt.WidgetWithChildrenShortcut)
-        close_find.setStatusTip(gettext("Close find"))
-        self.connect(self.show_action,
-                     QtCore.SIGNAL("toggled (bool)"),
-                     self.show_action_toggle)
-        self.connect(close_find,
-                     QtCore.SIGNAL("triggered(bool)"),
-                     self.close_triggered)
-        self.connect(self.find_text,
-                     QtCore.SIGNAL("textChanged(QString)"),
-                     self.find_text_changed)
-        self.connect(next,
-                     QtCore.SIGNAL("triggered(bool)"),
-                     self.find_next)
-        self.connect(prev,
-                     QtCore.SIGNAL("triggered(bool)"),
-                     self.find_prev)
-        self.connect(self.case_sensitive,
-                     QtCore.SIGNAL("stateChanged(int)"),
-                     self.find_text_changed)
-        self.connect(self.whole_words,
-                     QtCore.SIGNAL("stateChanged(int)"),
-                     self.find_text_changed)
-        self.connect(self.find_text,
-                     QtCore.SIGNAL("returnPressed()"),
-                     self.find_next)        
-        
-    def show_action_toggle(self, state):
-        self.setVisible(state)
-        if state:
-            self.find_text.setFocus()
-        else:
-            self.find_text.setText('')
-    
-    def close_triggered(self, state):
-        self.show_action.setChecked(False)
-    
-    def find_text_changed(self, text):
-        self.find_avoid_moving()
-    
-    def find_get_flags(self):
-        flags = QtGui.QTextDocument.FindFlags()
-        if self.case_sensitive.isChecked():
-            flags = flags | QtGui.QTextDocument.FindCaseSensitively
-        if self.whole_words.isChecked():
-            flags = flags | QtGui.QTextDocument.FindWholeWords
-        return flags
-    
-    def find_avoid_moving(self):
-        self.find(self.text_edit.textCursor().selectionStart(), 0,
-                  self.find_get_flags())
-    
-    def find_next(self, state):
-        self.find(self.text_edit.textCursor().selectionEnd(), 0,
-                  self.find_get_flags())
-    
-    def find_prev(self, state):
-        self.find(self.text_edit.textCursor().selectionStart(),
-                  self.text_edit.document().characterCount(),
-                  self.find_get_flags() | QtGui.QTextDocument.FindBackward)
-    
-    def find(self, from_pos, restart_pos, flags):
-        doc = self.text_edit.document()
-        text = self.find_text.text()
-        cursor = doc.find(text, from_pos, flags)
-        if cursor.isNull():
-            # try again from the restart pos
-            cursor = doc.find(text, restart_pos, flags)
-        if cursor.isNull():
-            cursor = self.text_edit.textCursor()
-            cursor.setPosition(cursor.selectionStart())
-            self.text_edit.setTextCursor(cursor)
-            # Maybe make find_text background red like Firefox?
-        else:
-            self.text_edit.setTextCursor(cursor)
+    return QtGui.QIcon(":/%dx%d/%s.png" % (size, size, name))
 
 
 class InfoWidget(QtGui.QFrame):
@@ -1127,7 +1069,6 @@ class InfoWidget(QtGui.QFrame):
         self.setBackgroundRole(QtGui.QPalette.ToolTipBase) 
         self.setForegroundRole(QtGui.QPalette.ToolTipText)
 
-
 # Hackish test for monospace. Run bzr qcat lib/util.py to check.
 #888888888888888888888888888888888888888888888888888888888888888888888888888888
 #                                                                             8
@@ -1158,3 +1099,46 @@ def _get_monospace_font():
     font.setFixedPitch(True)
     return font
 
+def get_set_tab_width_chars(branch=None, tab_width_chars=None):
+    """Function to get the tab width in characters from the configuration.
+
+    @param branch: Use branch.conf as well as bazaar.conf if this is provided.
+    @param tab_width_chars: Number of characters to use as tab width: if branch
+        is provided, the tab width will be stored in branch.conf
+
+    Both arguments are optional, but if tab_width_chars is provided and branch is
+    not, nothing will be done.
+    
+    @return: Tab width, in characters.
+    """
+    if tab_width_chars is None:
+        config = get_branch_config(branch)
+        try:
+            tab_width_chars = int(config.get_user_option('tab_width'))
+            if tab_width_chars < 0:
+                raise TypeError("Invalid tab width")
+        except TypeError:
+            tab_width_chars = 8
+    else:
+        if branch:
+            branch.get_config().set_user_option("tab_width", tab_width_chars)
+
+    return tab_width_chars
+
+def get_tab_width_pixels(branch=None, tab_width_chars=None):
+    """Function to get the tab width in pixels based on a monospaced font.
+
+    If tab_width_chars is provided, it is simply converted to a value in pixels.  If
+    it is not provided, the configuration is retrieved from bazaar.conf.  If branch is
+    provided (and tab_width_chars is not), branch.conf is also checked.
+
+    @param tab_width_chars: Number of characters of tab width to convert to pixels.
+    @param branch: Branch to use when retrieving tab width from configuration.
+    
+    @return: Tab width, in pixels.
+    """
+    monospacedFont = get_monospace_font()
+    char_width = QtGui.QFontMetrics(monospacedFont).width(" ")
+    if tab_width_chars is None:
+        tab_width_chars = get_set_tab_width_chars(branch=branch)
+    return char_width*tab_width_chars
